import { useState } from "preact/hooks";
import { fetchJson } from "../../util/fetchJson.ts";
import { Loader } from "../../components/loader.tsx";
import { Modal } from "../../components/modal.tsx";
import useSwr from "swr";

export const NamespaceModal = ({ close }: { close: () => void }) => {
  const { data: namespaces, isLoading } = useSwr<string[]>(
    "/api/pack/namespaces",
    fetchJson,
  );
  const [namespace, setNamespace] = useState("");
  const [invalid, setInvalid] = useState("");

  const createNamespace = async (ns?: string) => {
    const res = await fetch("/api/pack/namespaces", {
      method: "POST",
      body: ns ?? namespace,
    });

    if (res.status === 200) {
      return close();
    }

    setInvalid(await res.text());
  };

  return (
    <Modal>
      {isLoading
        ? <Loader msg="Checking existing namespaces..." />
        : (
          <div class="flex flex-col gap-2">
            <p>Create a new namespace</p>
            {!namespaces?.includes("minecraft") &&
              (
                <button
                  class="w-full"
                  onClick={() => createNamespace("minecraft")}
                >
                  Create default minecraft namespace
                </button>
              )}
            <form
              onSubmit={(e) => {
                e.preventDefault();
                createNamespace();
              }}
              class="flex gap-2"
            >
              <input
                type="text"
                value={namespace}
                onInput={(e) => setNamespace((e.target as any).value)}
              />
              <button type="submit">Create</button>
            </form>
          </div>
        )}
    </Modal>
  );
};
